@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2020-2022 The btclib developers
+# Copyright (C) The btclib developers
 #
 # This file is part of btclib. It is subject to the license terms in the
 # LICENSE file found in the top-level directory of this distribution.
@@ -8,11 +8,11 @@
 # No part of btclib including this file, may be copied, modified, propagated,
 # or distributed except according to the terms contained in the LICENSE file.
 
-"__init__ module for the btclib package."
+"""__init__ module for the btclib package."""
 
 name = "btclib"
-__version__ = "2022.7.20"
+__version__ = "2023.2.3"
 __author__ = "The btclib developers"
 __author_email__ = "devs@btclib.org"
-__copyright__ = "Copyright (C) 2017-2022 The btclib developers"
+__copyright__ = "Copyright (C) 2017-2023 The btclib developers"
 __license__ = "MIT License"
