@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2020-2022 The btclib developers
+# Copyright (C) The btclib developers
 #
 # This file is part of btclib. It is subject to the license terms in the
 # LICENSE file found in the top-level directory of this distribution.
@@ -8,4 +8,10 @@
 # No part of btclib including this file, may be copied, modified, propagated,
 # or distributed except according to the terms contained in the LICENSE file.
 
-"""btclib.ecc submodule."""
+"""Module btclib.ecc."""
+
+from btclib.ecc.bip340_nonce import bip340_nonce_
+from btclib.ecc.dh import ansi_x9_63_kdf, diffie_hellman
+from btclib.ecc.pedersen import second_generator
+
+__all__ = ["bip340_nonce_", "second_generator", "ansi_x9_63_kdf", "diffie_hellman"]
