@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2020-2022 The btclib developers
+# Copyright (C) The btclib developers
 #
 # This file is part of btclib. It is subject to the license terms in the
 # LICENSE file found in the top-level directory of this distribution.
@@ -12,13 +12,18 @@
 
 https://github.com/bitcoin/bips/blob/master/bip-0174.mediawiki
 """
+from __future__ import annotations
+
 import base64
+import random
 from copy import deepcopy
 from dataclasses import dataclass
-from typing import Any, Dict, List, Mapping, Optional, Sequence, Type, Union
+from typing import Any, Callable, Mapping, Sequence, TypeVar
 
 from btclib.alias import Octets, String
-from btclib.bip32.key_origin import (
+from btclib.bip32 import (
+    BIP32KeyOrigin,
+    HdKeyPaths,
     assert_valid_hd_key_paths,
     decode_from_bip32_derivs,
     decode_hd_key_paths,
@@ -26,7 +31,7 @@ from btclib.bip32.key_origin import (
 )
 from btclib.exceptions import BTClibValueError
 from btclib.hashes import hash160, sha256
-from btclib.psbt.psbt_in import BIP32KeyOrigin, HdKeyPaths, PsbtIn, Tx, Witness
+from btclib.psbt.psbt_in import PsbtIn
 from btclib.psbt.psbt_out import PsbtOut
 from btclib.psbt.psbt_utils import (
     assert_valid_unknown,
@@ -39,8 +44,8 @@ from btclib.psbt.psbt_utils import (
     serialize_dict_bytes_bytes,
     serialize_hd_key_paths,
 )
-from btclib.script.script import serialize
-from btclib.script.script_pub_key import type_and_payload
+from btclib.script import Witness, serialize, type_and_payload
+from btclib.tx import Tx, join_txs
 from btclib.utils import bytesio_from_binarydata
 
 PSBT_MAGIC_BYTES = b"psbt"
@@ -57,7 +62,6 @@ PSBT_GLOBAL_VERSION = b"\xfb"
 
 
 def _assert_valid_version(version: int) -> None:
-
     # must be a 4-bytes int
     if not 0 <= version <= 0xFFFFFFFF:
         raise BTClibValueError(f"invalid version: {version}")
@@ -69,11 +73,11 @@ def _assert_valid_version(version: int) -> None:
 @dataclass
 class Psbt:
     tx: Tx
-    inputs: List[PsbtIn]
-    outputs: List[PsbtOut]
+    inputs: list[PsbtIn]
+    outputs: list[PsbtOut]
     version: int
     hd_key_paths: HdKeyPaths
-    unknown: Dict[bytes, bytes]
+    unknown: dict[bytes, bytes]
 
     def __init__(
         self,
@@ -82,10 +86,9 @@ class Psbt:
         outputs: Sequence[PsbtOut],
         version: int,
         hd_key_paths: Mapping[Octets, BIP32KeyOrigin],
-        unknown: Optional[Mapping[Octets, Octets]] = None,
+        unknown: Mapping[Octets, Octets] | None = None,
         check_validity: bool = True,
     ) -> None:
-
         self.tx = tx
         self.inputs = list(inputs)
         self.outputs = list(outputs)
@@ -97,8 +100,7 @@ class Psbt:
             self.assert_valid()
 
     def assert_valid(self) -> None:
-        "Assert logical self-consistency."
-
+        """Assert logical self-consistency."""
         self.tx.assert_valid()
 
         # ensure a non-null tx has been included
@@ -138,15 +140,14 @@ class Psbt:
         assert_valid_unknown(self.unknown)
 
     def assert_signable(self) -> None:
-
         self.assert_valid()
 
         for i, tx_in in enumerate(self.tx.vin):
-
             non_witness_utxo = self.inputs[i].non_witness_utxo
-            witness_utxo = self.inputs[i].witness_utxo
             redeem_script = self.inputs[i].redeem_script
-
+            # with python>=3.8 use walrus operator
+            # if witness_utxo := self.inputs[i].witness_utxo:
+            witness_utxo = self.inputs[i].witness_utxo
             if witness_utxo:
                 script_pub_key = witness_utxo.script_pub_key
                 script_type, payload = type_and_payload(script_pub_key.script)
@@ -172,8 +173,7 @@ class Psbt:
                 if payload != sha256(self.inputs[i].witness_script):
                     raise BTClibValueError("invalid witness script sha256")
 
-    def to_dict(self, check_validity: bool = True) -> Dict[str, Any]:
-
+    def to_dict(self, check_validity: bool = True) -> dict[str, Any]:
         if check_validity:
             self.assert_valid()
 
@@ -188,26 +188,24 @@ class Psbt:
 
     @classmethod
     def from_dict(
-        cls: Type["Psbt"], dict_: Mapping[str, Any], check_validity: bool = True
-    ) -> "Psbt":
-
+        cls: type[Psbt], dict_: Mapping[str, Any], check_validity: bool = True
+    ) -> Psbt:
         return cls(
             Tx.from_dict(dict_["tx"]),
             [PsbtIn.from_dict(psbt_in, False) for psbt_in in dict_["inputs"]],
             [PsbtOut.from_dict(psbt_out, False) for psbt_out in dict_["outputs"]],
             dict_["version"],
             # FIXME
-            decode_from_bip32_derivs(dict_["bip32_derivs"]),  # type: ignore
+            decode_from_bip32_derivs(dict_["bip32_derivs"]),  # type: ignore[arg-type]
             dict_["unknown"],
             check_validity,
         )
 
     def serialize(self, check_validity: bool = True) -> bytes:
-
         if check_validity:
             self.assert_valid()
 
-        psbt_bin: List[bytes] = [PSBT_MAGIC_BYTES, PSBT_SEPARATOR]
+        psbt_bin: list[bytes] = [PSBT_MAGIC_BYTES, PSBT_SEPARATOR]
 
         temp = self.tx.serialize(include_witness=False)
         psbt_bin.append(serialize_bytes(PSBT_GLOBAL_UNSIGNED_TX, temp))
@@ -220,27 +218,22 @@ class Psbt:
             psbt_bin.append(serialize_dict_bytes_bytes(b"", self.unknown))
 
         psbt_bin.append(PSBT_DELIMITER)
-        for input_map in self.inputs:
-            psbt_bin.append(input_map.serialize() + b"\x00")
-        for output_map in self.outputs:
-            psbt_bin.append(output_map.serialize() + b"\x00")
+        psbt_bin.extend(input_map.serialize() + b"\x00" for input_map in self.inputs)
+        psbt_bin.extend(output_map.serialize() + b"\x00" for output_map in self.outputs)
         return b"".join(psbt_bin)
 
     @classmethod
-    def parse(
-        cls: Type["Psbt"], psbt_bin: Octets, check_validity: bool = True
-    ) -> "Psbt":
-        "Return a Psbt by parsing binary data."
-
-        # FIXME: psbt_bin should be BinaryData
+    def parse(cls: type[Psbt], psbt_bin: Octets, check_validity: bool = True) -> Psbt:
+        """Return a Psbt by parsing binary data."""
+        # FIXME psbt_bin should be BinaryData
         # stream = bytesio_from_binarydata(psbt_bin)
         # and the deserialization should happen reading the stream
         # not slicing bytes
 
         tx = Tx(check_validity=False)
         version = 0
-        hd_key_paths: Dict[Octets, BIP32KeyOrigin] = {}
-        unknown: Dict[Octets, Octets] = {}
+        hd_key_paths: dict[Octets, BIP32KeyOrigin] = {}
+        unknown: dict[Octets, Octets] = {}
 
         # psbt_bin = bytes_from_octets(psbt_bin)
         stream = bytesio_from_binarydata(psbt_bin)
@@ -261,12 +254,12 @@ class Psbt:
             else:  # unknown
                 unknown[k] = v
 
-        inputs: List[PsbtIn] = []
+        inputs: list[PsbtIn] = []
         for _ in tx.vin:
             input_map, stream = deserialize_map(stream)
             inputs.append(PsbtIn.parse(input_map))
 
-        outputs: List[PsbtOut] = []
+        outputs: list[PsbtOut] = []
         for _ in tx.vout:
             output_map, stream = deserialize_map(stream)
             outputs.append(PsbtOut.parse(output_map))
@@ -287,20 +280,17 @@ class Psbt:
 
     @classmethod
     def b64decode(
-        cls: Type["Psbt"], psbt_str: String, check_validity: bool = True
-    ) -> "Psbt":
-
+        cls: type[Psbt], psbt_str: String, check_validity: bool = True
+    ) -> Psbt:
         if isinstance(psbt_str, str):
             psbt_str = psbt_str.strip()
 
         psbt_decoded = base64.b64decode(psbt_str)
 
-        # pylance cannot grok the following line
-        return cls.parse(psbt_decoded, check_validity)  # type: ignore
+        return cls.parse(psbt_decoded, check_validity)
 
     @classmethod
-    def from_tx(cls: Type["Psbt"], tx: Tx, check_validity: bool = True) -> "Psbt":
-
+    def from_tx(cls: type[Psbt], tx: Tx, check_validity: bool = True) -> Psbt:
         for tx_in in tx.vin:
             tx_in.script_sig = b""
             tx_in.script_witness = Witness()
@@ -308,8 +298,8 @@ class Psbt:
         outputs = [PsbtOut() for _ in tx.vout]
 
         psbt_version = 0
-        hd_key_paths: Dict[Octets, BIP32KeyOrigin] = {}
-        unknown: Dict[Octets, Octets] = {}
+        hd_key_paths: dict[Octets, BIP32KeyOrigin] = {}
+        unknown: dict[Octets, Octets] = {}
 
         return cls(
             tx,
@@ -321,11 +311,30 @@ class Psbt:
             check_validity,
         )
 
+    def sort_inputs(self, ordering_func: Callable[[PsbtIn], int] | None = None) -> None:
+        """Sort psbt inputs.
+
+        sorting logic is ordering_func if present, shuffle otherwise.
+        """
+        self.inputs, self.tx.vin = _sort_or_shuffle_together(
+            self.inputs, self.tx.vin, ordering_func
+        )
+
+    def sort_outputs(
+        self, ordering_func: Callable[[PsbtOut], int] | None = None
+    ) -> None:
+        """Sort psbt outputs.
+
+        sorting logic is ordering_func if present, shuffle otherwise.
+        """
+        self.outputs, self.tx.vout = _sort_or_shuffle_together(
+            self.outputs, self.tx.vout, ordering_func
+        )
+
 
 def _combine_field(
-    psbt_map: Union[PsbtIn, PsbtOut, Psbt], out: Union[PsbtIn, PsbtOut, Psbt], key: str
+    psbt_map: PsbtIn | PsbtOut | Psbt, out: PsbtIn | PsbtOut | Psbt, key: str
 ) -> None:
-
     item = getattr(psbt_map, key)
     if not item:
         return
@@ -335,13 +344,17 @@ def _combine_field(
     elif attr != item:
         if isinstance(item, dict):
             attr.update(item)
-        # TODO: fails for final_script_witness
+        # TODO fails for final_script_witness
         # elif isinstance(item, list):
         #     additional_elements = [i for i in item if i not in attr]
         #     attr += additional_elements
 
 
 def combine_psbts(psbts: Sequence[Psbt]) -> Psbt:
+    """Merge Psbt data from multiple Psbts with same TxId.
+
+    Basically used to merge signatures.
+    """
     final_psbt = psbts[0]
     tx_id = psbts[0].tx.id
     for psbt in psbts[1:]:
@@ -350,7 +363,6 @@ def combine_psbts(psbts: Sequence[Psbt]) -> Psbt:
 
     final_psbt.version = max(psbt.version for psbt in psbts)
     for psbt in psbts[1:]:
-
         for i, inp in enumerate(final_psbt.inputs):
             _combine_field(psbt.inputs[i], inp, "non_witness_utxo")
             _combine_field(psbt.inputs[i], inp, "witness_utxo")
@@ -395,14 +407,14 @@ def finalize_psbt(psbt: Psbt) -> Psbt:
     """
     psbt = deepcopy(psbt)
     psbt.assert_valid()
-    # TODO: finalizers must fail to finalize inputs
+    # TODO finalizers must fail to finalize inputs
     # which have signatures that do not match the specified sign_ type
     for psbt_in in psbt.inputs:
         if not psbt_in.partial_sigs:
             raise BTClibValueError("missing signatures")
         sigs = psbt_in.partial_sigs.values()
         # https://github.com/bitcoin/bips/blob/master/bip-0147.mediawiki#motivation
-        cmds: List[bytes] = [b""] if len(sigs) > 1 else []
+        cmds: list[bytes] = [b""] if len(sigs) > 1 else []
         cmds += sigs
         if psbt_in.witness_script:
             psbt_in.final_script_sig = serialize([psbt_in.redeem_script])
@@ -418,12 +430,13 @@ def finalize_psbt(psbt: Psbt) -> Psbt:
 
 
 def extract_tx(psbt: Psbt, check_validity: bool = True) -> Tx:
-    """Extract the Tx fro the Psbt
+    """Extract the Tx fro the Psbt.
 
     The Transaction Extractor must only accept a PSBT.
     It checks whether all inputs have complete scriptSigs
     and scriptWitnesses by checking for the presence of
-    0x07 Finalized scriptSig and 0x08 Finalized scriptWitness typed records.
+    0x07 Finalized scriptSig and 0x08 Finalized scriptWitness typed
+    records.
 
     If they do, the Transaction Extractor should construct
     complete scriptSigs and scriptWitnesses and encode them
@@ -439,7 +452,6 @@ def extract_tx(psbt: Psbt, check_validity: bool = True) -> Tx:
     However it may be able to in order to validate
     the network serialized transaction at the same time.
     """
-
     if check_validity:
         psbt.assert_valid()
 
@@ -452,3 +464,111 @@ def extract_tx(psbt: Psbt, check_validity: bool = True) -> Tx:
     if check_validity:
         tx.assert_valid()
     return tx
+
+
+TypeA = TypeVar("TypeA")
+TypeB = TypeVar("TypeB")
+
+
+def _sort_or_shuffle_together(
+    sequence_a: Sequence[TypeA],
+    sequence_b: Sequence[TypeB],
+    ordering_func: Callable[[TypeA], int] | None = None,
+) -> tuple[list[TypeA], list[TypeB]]:
+    """Sort together with ordering_func if provided, else shuffle together.
+
+    Sort is on TypeA, both sequences must have same length.
+    """
+    if len(sequence_a) != len(sequence_b):
+        raise BTClibValueError("sequences must have same length")
+
+    tmp = list(zip(sequence_a, sequence_b))
+    if ordering_func is None:
+        random.shuffle(tmp)
+    else:
+        tmp.sort(key=lambda t: ordering_func(t[0]))  # type: ignore
+    tuple_a, tuple_b = zip(*tmp)
+    return list(tuple_a), list(tuple_b)
+
+
+def _ensure_consistency(psbts: Sequence[Psbt]) -> None:
+    """Check validity of each psbt and conflicts in key_paths or unknown."""
+    key_paths: dict[bytes, BIP32KeyOrigin] = {}
+    r_key_paths: dict[BIP32KeyOrigin, bytes] = {}
+    unknown: dict[bytes, bytes] = {}
+    for psbt in psbts:
+        psbt.assert_valid()
+
+        if any(
+            pub_key in key_paths and key_origin != key_paths[pub_key]
+            for pub_key, key_origin in psbt.hd_key_paths.items()
+        ):
+            raise BTClibValueError("hd_key_paths: same pub_key, different key_origin")
+        key_paths.update(psbt.hd_key_paths)
+
+        if any(
+            key_origin in r_key_paths and pub_key != r_key_paths[key_origin]
+            for pub_key, key_origin in psbt.hd_key_paths.items()
+        ):
+            raise BTClibValueError("hd_key_paths: same key_origin, different pub_key")
+        r_key_paths.update(
+            {key_origin: pub_key for pub_key, key_origin in psbt.hd_key_paths.items()}
+        )
+
+        if any(
+            key in unknown and value != unknown[key]
+            for key, value in psbt.unknown.items()
+        ):
+            raise BTClibValueError("unknown: same key, different value")
+        unknown.update(psbt.unknown)
+
+
+def join_psbts(
+    psbts: Sequence[Psbt],
+    enforce_same_tx_version: bool,
+    enforce_same_tx_lock_time: bool,
+    merge_out: bool,
+    shuffle_inp: bool,
+    shuffle_out: bool,
+    sort_inp: Callable[[PsbtIn], int] | None = None,
+    sort_out: Callable[[PsbtOut], int] | None = None,
+) -> Psbt:
+    """Join multiple psbts into a single one by merging inputs and outputs.
+
+    inputs/outputs are shuffled by default. If shuffle_{in|out}=False,
+    they are simply concatenated in the same order as psbts are
+    specified. A specific ordering can be specified via sort_{inp|out},
+    which overwrite shuffle when present.
+    """
+    _ensure_consistency(psbts)
+
+    inputs = [inp for psbt in psbts for inp in psbt.inputs]
+    outputs = [outp for psbt in psbts for outp in psbt.outputs]
+    hd_key_paths: dict[Octets, BIP32KeyOrigin] = {
+        k: v for psbt in psbts for k, v in psbt.hd_key_paths.items()
+    }
+    unknown: dict[Octets, Octets] = {
+        k: v for psbt in psbts for k, v in psbt.unknown.items()
+    }
+    version = max(psbt.version for psbt in psbts)
+
+    merged_tx = join_txs(
+        [psbt.tx for psbt in psbts],
+        enforce_same_tx_version,
+        enforce_same_tx_lock_time,
+        False,
+        False,
+        False,
+    )
+
+    psbt = Psbt(merged_tx, inputs, outputs, version, hd_key_paths, unknown)
+    if shuffle_inp or sort_inp:
+        psbt.sort_inputs(sort_inp)
+    if shuffle_out or sort_out:
+        psbt.sort_outputs(sort_out)
+    if merge_out:
+        # TODO is it ok to merge outputs after sorting?
+        raise BTClibValueError("output merge not implemented yet")
+
+    psbt.assert_valid()
+    return psbt
