@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2020-2022 The btclib developers
+# Copyright (C) The btclib developers
 #
 # This file is part of btclib. It is subject to the license terms in the
 # LICENSE file found in the top-level directory of this distribution.
@@ -8,4 +8,11 @@
 # No part of btclib including this file, may be copied, modified, propagated,
 # or distributed except according to the terms contained in the LICENSE file.
 
-"""btclib.tx submodule."""
+"""Module btclib.tx."""
+
+from btclib.tx.out_point import OutPoint
+from btclib.tx.tx import Tx, join_txs
+from btclib.tx.tx_in import TxIn
+from btclib.tx.tx_out import TxOut
+
+__all__ = ["OutPoint", "TxOut", "TxIn", "Tx", "join_txs"]
