@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2020-2022 The btclib developers
+# Copyright (C) The btclib developers
 #
 # This file is part of btclib. It is subject to the license terms in the
 # LICENSE file found in the top-level directory of this distribution.
@@ -8,7 +8,7 @@
 # No part of btclib including this file, may be copied, modified, propagated,
 # or distributed except according to the terms contained in the LICENSE file.
 
-"Varbytes encoding and decoding functions."
+"""Varbytes encoding and decoding functions."""
 
 from btclib import var_int
 from btclib.alias import BinaryData, Octets
@@ -18,7 +18,6 @@ from btclib.utils import bytes_from_octets, bytesio_from_binarydata
 
 def parse(stream: BinaryData, forbid_zero_size: bool = False) -> bytes:
     """Return the variable-length octets read from a stream."""
-
     stream = bytesio_from_binarydata(stream)
     i = var_int.parse(stream)
     if forbid_zero_size and i == 0:
@@ -31,7 +30,6 @@ def parse(stream: BinaryData, forbid_zero_size: bool = False) -> bytes:
 
 
 def serialize(octets: Octets) -> bytes:
-    "Return the var_int(len(octets)) + octets serialization of octets."
-
+    """Return the var_int(len(octets)) + octets serialization of octets."""
     bytes_ = bytes_from_octets(octets)
     return var_int.serialize(len(bytes_)) + bytes_
