@@ -1,6 +1,6 @@
 #!/usr/bin/env python3
 
-# Copyright (C) 2017-2022 The btclib developers
+# Copyright (C) The btclib developers
 #
 # This file is part of btclib. It is subject to the license terms in the
 # LICENSE file found in the top-level directory of this distribution.
@@ -8,19 +8,21 @@
 # No part of btclib including this file, may be copied, modified, propagated,
 # or distributed except according to the terms contained in the LICENSE file.
 
-"Tests for the `btclib.script.script_pub_key` module."
+"""Tests for the `btclib.script.script_pub_key` module."""
+
+from __future__ import annotations
 
 import json
 from os import path
-from typing import List
 
 import pytest
 
 from btclib import b32, b58, var_bytes
 from btclib.exceptions import BTClibValueError
 from btclib.hashes import hash160, sha256
-from btclib.script.script import Command, Script, parse, serialize
-from btclib.script.script_pub_key import (
+from btclib.script import (
+    Command,
+    Script,
     ScriptPubKey,
     address,
     assert_p2ms,
@@ -32,10 +34,11 @@ from btclib.script.script_pub_key import (
     assert_p2wsh,
     is_nulldata,
     is_p2ms,
+    output_pubkey,
+    parse,
+    serialize,
     type_and_payload,
 )
-from btclib.script.taproot import output_pubkey
-from btclib.to_pub_key import Key
 
 
 def test_eq() -> None:
@@ -58,7 +61,6 @@ def test_unknown_network() -> None:
 
 
 def test_nulldata() -> None:
-
     OP_RETURN = b"\x6a"  # pylint: disable=invalid-name
 
     # self-consistency
@@ -107,7 +109,6 @@ def test_nulldata() -> None:
 
 
 def test_nulldata2() -> None:
-
     for length in (0, 1, 16, 17, 74, 75, 76, 77, 78, 79, 80):
         payload = b"\x00" * length
         script_pub_key = serialize(["OP_RETURN", payload])
@@ -115,7 +116,6 @@ def test_nulldata2() -> None:
 
 
 def test_nulldata3() -> None:
-
     err_msg = "invalid nulldata payload length: "
     with pytest.raises(BTClibValueError, match=err_msg):
         payload = "0A" * 81
@@ -146,8 +146,7 @@ def test_nulldata3() -> None:
 
 
 def test_nulldata4() -> None:
-
-    script_: List[Command] = [
+    script_: list[Command] = [
         "OP_RETURN",
         "OP_RETURN",
         "OP_3",
@@ -156,18 +155,17 @@ def test_nulldata4() -> None:
         "OP_0",
         "OP_3",
     ]
-    # FIXME: serialization is not 0x6A{1 byte data-length}{data 6 bytes)}
+    # FIXME serialization is not 0x6A{1 byte data-length}{data 6 bytes)}
     script_pub_key = serialize(script_)
     assert len(script_pub_key) == 7
     assert parse(script_pub_key) == script_
     script_type, _ = type_and_payload(script_pub_key)
-    # FIXME: it should be "nulldata"
+    # FIXME it should be "nulldata"
     assert script_type == "unknown"
     # assert is_nulldata(script_pub_key)
 
 
 def test_p2pk() -> None:
-
     # self-consistency
     pub_key = "02 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf"
     script_pub_key = serialize([pub_key, "OP_CHECKSIG"])
@@ -207,7 +205,6 @@ def test_p2pk() -> None:
 
 
 def test_p2pkh() -> None:
-
     # self-consistency
     pub_key = (
         "04 "
@@ -255,7 +252,6 @@ def test_p2pkh() -> None:
 
 
 def test_p2wpkh() -> None:
-
     # self-consistency
     pub_key = "02 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf"
     payload = hash160(pub_key)
@@ -288,7 +284,6 @@ def test_p2wpkh() -> None:
 
 
 def test_p2sh() -> None:
-
     # self-consistency
     pub_key = "02 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf"
     redeem_script = ScriptPubKey.p2pkh(pub_key).script
@@ -331,7 +326,6 @@ def test_p2sh() -> None:
 
 
 def test_p2wsh() -> None:
-
     # self-consistency
     pub_key = "02 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf"
     redeem_script = ScriptPubKey.p2pkh(pub_key).script
@@ -365,35 +359,36 @@ def test_p2wsh() -> None:
 
 
 def test_unknown() -> None:
-
     script_pub_key = serialize(["OP_16", 20 * b"\x00"])
     assert address(script_pub_key) == ""
     assert type_and_payload(script_pub_key) == ("unknown", script_pub_key)
 
 
 def test_p2ms_1() -> None:
-
     # self-consistency
+    # documented test case: https://learnmeabitcoin.com/guide/p2ms
     pub_key0 = "04 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf f7d8a473e7e2e6d317b87bafe8bde97e3cf8f065dec022b51d11fcdd0d348ac4"
     pub_key1 = "04 61cbdcc5409fb4b4d42b51d33381354d80e550078cb532a34bfa2fcfdeb7d765 19aecc62770f5b0e4ef8551946d8a540911abe3e7854a26f39f58b25c15342af"
-
-    # documented test case: https://learnmeabitcoin.com/guide/p2ms
-    script_pub_key = bytes.fromhex(  # fmt: off
+    # sourcery skip: move-assign-in-block
+    script_pub_key = bytes.fromhex(
         "51"  # OP_1
         "41"  # canonical 65-bytes push
-        + pub_key0
-        + "41"  # noqa E148  # canonical 65-bytes push
-        + pub_key1
-        + "52"  # noqa E148  # OP_2
+        f"{pub_key0}"
+        "41"  # canonical 65-bytes push
+        f"{pub_key1}"
+        "52"  # OP_2
         "ae"  # OP_CHECKMULTISIG
-    )  # fmt: on
+    )
     assert is_p2ms(script_pub_key)
     assert address(script_pub_key) == ""
     script_type, payload = type_and_payload(script_pub_key)
     assert script_type == "p2ms"
     assert payload == script_pub_key[:-1]
-    pub_keys: List[Key] = [pub_key0, pub_key1]
-    assert script_pub_key == ScriptPubKey.p2ms(1, pub_keys, lexi_sort=False).script
+    pub_keys = [pub_key0, pub_key1]
+    assert (
+        script_pub_key
+        == ScriptPubKey.p2ms(1, pub_keys, lexicographic_sorting=False).script
+    )
 
     err_msg = "invalid m in m-of-n: "
     with pytest.raises(BTClibValueError, match=err_msg):
@@ -401,7 +396,7 @@ def test_p2ms_1() -> None:
     err_msg = "invalid n in m-of-n: "
     with pytest.raises(BTClibValueError, match=err_msg):
         # pylance cannot grok the following line
-        ScriptPubKey.p2ms(4, [pub_key0] * 17)  # type: ignore
+        ScriptPubKey.p2ms(4, [pub_key0] * 17)
     err_msg = "invalid m in m-of-n: "
     with pytest.raises(BTClibValueError, match=err_msg):
         ScriptPubKey.p2ms(0, pub_keys)
@@ -413,7 +408,7 @@ def test_p2ms_1() -> None:
     with pytest.raises(BTClibValueError, match=err_msg):
         ScriptPubKey.p2ms(1, [f"{pub_key0}00", pub_key1])
 
-    script_: List[Command] = [
+    script_: list[Command] = [
         "OP_1",
         f"{pub_key0}00",
         pub_key1,
@@ -463,23 +458,24 @@ def test_p2ms_1() -> None:
 
 
 def test_p2ms_2() -> None:
-
     m = 1
 
     # all uncompressed
     pub_key0 = "04 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf f7d8a473e7e2e6d317b87bafe8bde97e3cf8f065dec022b51d11fcdd0d348ac4"
     pub_key1 = "04 61cbdcc5409fb4b4d42b51d33381354d80e550078cb532a34bfa2fcfdeb7d765 19aecc62770f5b0e4ef8551946d8a540911abe3e7854a26f39f58b25c15342af"
     pub_key2 = "04 79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798 483ADA7726A3C4655DA4FBFC0E1108A8FD17B448A68554199C47D08FFB10D4B8"
-    uncompressed_pub_keys: List[Key] = [pub_key0, pub_key1, pub_key2]
+    uncompressed_pub_keys = [pub_key0, pub_key1, pub_key2]
     # mixed compressed / uncompressed public keys
     pub_key0 = "04 cc71eb30d653c0c3163990c47b976f3fb3f37cccdcbedb169a1dfef58bbfbfaf f7d8a473e7e2e6d317b87bafe8bde97e3cf8f065dec022b51d11fcdd0d348ac4"
     pub_key1 = "03 61cbdcc5409fb4b4d42b51d33381354d80e550078cb532a34bfa2fcfdeb7d765"
     pub_key2 = "02 79BE667EF9DCBBAC55A06295CE870B07029BFCDB2DCE28D959F2815B16F81798"
-    mixed_pub_keys: List[Key] = [pub_key0, pub_key1, pub_key2]
+    mixed_pub_keys = [pub_key0, pub_key1, pub_key2]
 
     for pub_keys in (uncompressed_pub_keys, mixed_pub_keys):
-        for lexi_sort in (True, False):
-            script_pub_key = ScriptPubKey.p2ms(m, pub_keys, lexi_sort=lexi_sort).script
+        for lexicographic_sorting in (True, False):
+            script_pub_key = ScriptPubKey.p2ms(
+                m, pub_keys, lexicographic_sorting=lexicographic_sorting
+            ).script
             assert is_p2ms(script_pub_key)
             assert address(script_pub_key) == ""
             script_type, payload = type_and_payload(script_pub_key)
@@ -490,11 +486,11 @@ def test_p2ms_2() -> None:
 def test_p2ms_3() -> None:
     # tx_id 33ac2af1a6f894276713b59ed09ce1a20fed5b36d169f20a3fe831dc45564d57
     # output n 0
-    keys: List[Command] = [
+    keys: list[Command] = [
         "036D568125A969DC78B963B494FA7ED5F20EE9C2F2FC2C57F86C5DF63089F2ED3A",
         "03FE4E6231D614D159741DF8371FA3B31AB93B3D28A7495CDAA0CD63A2097015C7",
     ]
-    cmds: List[Command] = ["OP_1", *keys, "OP_2", "OP_CHECKMULTISIG"]
+    cmds: list[Command] = ["OP_1", *keys, "OP_2", "OP_CHECKMULTISIG"]
     script_pub_key = ScriptPubKey(serialize(cmds))
     assert script_pub_key == ScriptPubKey.p2ms(1, keys)
 
@@ -509,7 +505,7 @@ def test_p2ms_3() -> None:
 
     # tx 56214420a7c4dcc4832944298d169a75e93acf9721f00656b2ee0e4d194f9970
     # input n 1
-    cmds_sig: List[Command] = [
+    cmds_sig: list[Command] = [
         "OP_0",
         "3045022100dba1e9b1c8477fd364edcc1f81845928202daf465a1e2d92904c13c88761cbd002200add6af863dfdb7efb95f334baec041e90811ae9d81624f9f87f33a56761f29401",
     ]
@@ -518,15 +514,14 @@ def test_p2ms_3() -> None:
     # parse(serialize(*)) is to enforce same string case convention
     assert script.asm == parse(serialize(cmds_sig + cmds))
 
-    # TODO: evaluate
+    # TODO evaluate
 
 
 def test_bip67() -> None:
-    "BIP67 test vectors https://en.bitcoin.it/wiki/BIP_0067"
-
+    """BIP67 test vectors https://en.bitcoin.it/wiki/BIP_0067."""
     data_folder = path.join(path.dirname(__file__), "_data")
     filename = path.join(data_folder, "bip67_test_vectors.json")
-    with open(filename, "r", encoding="ascii") as file_:
+    with open(filename, encoding="ascii") as file_:
         # json.dump(test_vectors, f, indent=4)
         test_vectors = json.load(file_)
 
@@ -534,7 +529,7 @@ def test_bip67() -> None:
     for i in test_vectors:
         keys, addr = test_vectors[i]
 
-        script_pub_key = ScriptPubKey.p2ms(m, keys, lexi_sort=True).script
+        script_pub_key = ScriptPubKey.p2ms(m, keys, lexicographic_sorting=True).script
         assert is_p2ms(script_pub_key)
         assert address(script_pub_key) == ""
         script_type, payload = type_and_payload(script_pub_key)
@@ -545,14 +540,13 @@ def test_bip67() -> None:
 
 
 def test_non_standard_script_in_p2wsh() -> None:
-
     network = "mainnet"
 
     addr = "bc1qqst9un5sz8576fy2nnqkpm4rpfh0weveqwtt8zxgjp02g2mx5q7s2vresu"
     script_pub_key = ScriptPubKey.from_address(addr)
     assert addr == address(script_pub_key.script, network)
 
-    fed_pub_keys: List[Command] = [
+    fed_pub_keys: list[Command] = [
         "03356aeda9c56586fe1e4a63d5118ffa3bf29bd91c6323e31de113a500a1ffe441".upper(),
         "0339f970e066a3efe1787722bd9cd59f69f1cf5cd29cb39fdec845415d8dbcb7a6".upper(),
         "0346f1233885981cc50b4064b6ed27174a149ac0842a25941f280302ddd7d2153d".upper(),
@@ -561,19 +555,19 @@ def test_non_standard_script_in_p2wsh() -> None:
         "03a88488e5ab6ae35c7d22d76efb9578e4a71b59c2ff0bd3cc3277e3a60717f3d6".upper(),
     ]
 
-    rec_pub_keys: List[Command] = [
+    rec_pub_keys: list[Command] = [
         "02219c3f199942fdd37a88065a8a8333189aadb5667a7c27681f66952fddf0eea4".upper(),
         "034c52cdf0125e50a53556c3f7586245f3f556bf26a80a4dae0ea6d0c81c11ebef".upper(),
         "03e8abfc4e3dcd5be461e79c9fa68a4d657b344391d7fd65ed40aaa56f584c7711".upper(),
     ]
 
     # fmt: off
-    redeem_script_cmds: List[Command] = [
+    redeem_script_cmds: list[Command] = [
         "OP_IF",
             "OP_3", *fed_pub_keys, "OP_6", "OP_CHECKMULTISIG",  # noqa E131
         "OP_ELSE",
-            5184, "OP_CHECKSEQUENCEVERIFY", "OP_DROP",  # noqa E131
-            "OP_2", *rec_pub_keys, "OP_3", "OP_CHECKMULTISIG",  # noqa E131
+            5184, "OP_CHECKSEQUENCEVERIFY", "OP_DROP",  # E131
+            "OP_2", *rec_pub_keys, "OP_3", "OP_CHECKMULTISIG",  # E131
         "OP_ENDIF",
     ]
     # fmt: on
